@@ -45,6 +45,7 @@
 #include "jazdrive_v_in.xpm"
 #include "jazdrive_v_out.xpm"
 
+
 /*
  *-------------------------------------------------------------------------
  * icon struct
@@ -138,8 +139,6 @@ static void about_cb (AppletWidget *widget, gpointer data)
 	gtk_widget_show (about);
 
 	return;
-	widget = NULL;
-	data = NULL;
 }
 
 static void browse_cb (AppletWidget *widget, gpointer data)
@@ -171,7 +170,6 @@ static void browse_cb (AppletWidget *widget, gpointer data)
 	}
 
 	return;
-	widget = NULL;
 }
 
 /*
@@ -184,7 +182,7 @@ static gint device_is_in_mountlist(DriveData *dd)
 {
 	FILE *fp;
 	gchar *command_line = "mount";
-	gchar buf[201];
+	gchar buf[256];
 	gint found = FALSE;
 
 	if (!dd->mount_point) return FALSE;
@@ -443,32 +441,35 @@ static gint drive_update_cb(gpointer data)
 static gint mount_cb(GtkWidget *widget, gpointer data)
 {
 	DriveData *dd = data;
-	gchar command_line[300];
-	gchar buf[200];
+	gchar *command_line;
+	gchar buf[512];
 	FILE *fp;
 	GString *str;
 	gint check = device_is_mounted(dd);
 
 	/* Stop the user from displaying zillions of error messages */
 	if (dd->error_dialog)
-	{
+		{
 		gdk_window_show(dd->error_dialog->window);
 		gdk_window_raise(dd->error_dialog->window);
 		return FALSE;
-	}
+		}
 
 	if (!check)
-		g_snprintf(command_line, sizeof(command_line),
-			   "mount %s 2>&1", dd->mount_point);
+		{
+		command_line = g_strdup_printf("mount %s 2>&1", dd->mount_point);
+		}
 	else
-		g_snprintf(command_line, sizeof(command_line),
-			   "umount %s 2>&1", dd->mount_point);
+		{
+		command_line = g_strdup_printf("umount %s 2>&1", dd->mount_point);
+		}
 
 	fp = popen(command_line, "r");
 
 	if (!fp)
 		{
 		printf("unable to run command: %s\n", command_line);
+		g_free(command_line);
 		return FALSE;
 		}
 
@@ -509,17 +510,19 @@ static gint mount_cb(GtkWidget *widget, gpointer data)
 		}
 
 	g_string_free(str, TRUE);
+	g_free(command_line);
+
 	return FALSE;
-	widget = NULL;
 }
 
 static void eject(DriveData *dd)
 {
-	gchar command_line[300];
-	gchar buffer[200];
-	gchar dn[200];	/* Devicename */
-	gchar mp[200];	/* Mountpoint */
+	gchar *command_line;
+	gchar buffer[512];
+	gchar dn[256];	/* Devicename */
+	gchar mp[256];	/* Mountpoint */
 	FILE *ml;	/* Mountlist */
+	gint found = FALSE;
 
 	if (!dd->mount_point) return;
 
@@ -533,40 +536,47 @@ static void eject(DriveData *dd)
 	
 	if (dd->mounted) {
 		ml = popen("mount", "r");
-		while (fgets(buffer, sizeof(buffer), ml)) {
-			sscanf(buffer, "%s %*s %s", dn, mp);
-			if (!strcmp(mp, dd->mount_point))
+		while (ml && fgets(buffer, sizeof(buffer), ml)) {
+			if (sscanf(buffer, "%255s %*s %255s", dn, mp) == 2 &&
+			    (mp && strcmp(mp, dd->mount_point) == 0)) {
+				found = TRUE;
 				break;
+			}
 		}
-		pclose (ml);
+		if (ml)
+			pclose (ml);
 	} else {
 		ml = fopen("/etc/fstab", "r");
-		while (fgets(buffer, sizeof(buffer), ml)) {
-			sscanf(buffer, "%s %s", dn, mp);
-			if (!strcmp(mp, dd->mount_point))
+		while (ml && fgets(buffer, sizeof(buffer), ml)) {
+			if (sscanf(buffer, "%255s %255s", dn, mp) == 2 &&
+			    strcmp(mp, dd->mount_point) == 0) {
+				found = TRUE;
 				break;
+			}
 		}
-		fclose (ml);
+		if (ml)
+			fclose (ml);
 	}
 	
-	if (strcmp(mp, dd->mount_point)) {	/* mp != dd->mount_point */
+	if (!found) {	/* mp != dd->mount_point */
 		printf("WARNING: drivemount.c ... dd->mount_point not found in list\
 			 (output of mount, or /etc/fstab) \n");
 		return;
 	}
 
 	if (dd->mounted) {
-		g_snprintf (command_line, sizeof(command_line),
-			    "eject -u '%s'", dn);
+		command_line = g_strdup_printf("eject -u '%s'", dn);
 		/* perhaps it doesn't like the -u option */
-		if(system (command_line)!=0) {
-			g_snprintf (command_line, sizeof(command_line),
-				    "eject '%s'", dn);
+		if(system(command_line) != 0) {
+			g_free(command_line);
+			command_line = g_strdup_printf("eject '%s'", dn);
 			gnome_execute_shell (NULL, command_line);
 		}
+		g_free(command_line);
 	} else {
-		g_snprintf (command_line, sizeof(command_line), "eject '%s'", dn);
+		command_line = g_strdup_printf("eject '%s'", dn);
 		gnome_execute_shell (NULL, command_line);
+		g_free(command_line);
 	}
 }
 
@@ -577,7 +587,6 @@ static void eject_cb(AppletWidget *applet, gpointer data)
 	eject(dd);
 
 	return;
-        applet = NULL;
 }
 
 /*
@@ -607,7 +616,6 @@ static void applet_change_orient(GtkWidget *w, PanelOrientType o, gpointer data)
 
 	redraw_pixmap(dd);
 	return;
-	w = NULL;
 }
 
 #ifdef HAVE_PANEL_PIXEL_SIZE
@@ -622,7 +630,6 @@ static void applet_change_pixel_size(GtkWidget *w, int size, gpointer data)
 
 	redraw_pixmap(dd);
 	return;
-	w = NULL;
 }
 #endif
 
@@ -631,8 +638,6 @@ static gint applet_save_session(GtkWidget *widget, gchar *privcfgpath, gchar *gl
 	DriveData *dd = data;
 	property_save(privcfgpath, dd);
 	return FALSE;
-	widget = NULL;
-	globcfgpath = NULL;
 }
 
 static void destroy_drive_widget(GtkWidget *widget, gpointer data)
@@ -647,7 +652,6 @@ static void destroy_drive_widget(GtkWidget *widget, gpointer data)
 	g_free(dd);
 
 	return;
-	widget = NULL;
 }
 
 static void
@@ -773,8 +777,6 @@ static GtkWidget * applet_start_new_applet(const gchar *goad_id, const gchar **p
 	dd = create_drive_widget(applet);
 
 	return applet;
-	params = NULL;
-	nparams = 0;
 }
 
 int main (int argc, char *argv[])
@@ -832,7 +834,6 @@ static void dnd_drag_begin_cb(GtkWidget *widget, GdkDragContext *context, gpoint
 				 GNOME_PIXMAP(dd->button_pixmap)->pixmap, NULL,
 				 -5, -5);
 	return;
-	widget = NULL;
 }
 
 static void dnd_set_data_cb(GtkWidget *widget, GdkDragContext *context,
@@ -863,9 +864,6 @@ static void dnd_set_data_cb(GtkWidget *widget, GdkDragContext *context,
 					8, NULL, 0);
 		}
 	return;
-	widget = NULL;
-	context = NULL;
-	time = 0;
 }
 
 static void dnd_init(DriveData *dd)
