@@ -31,12 +31,13 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "ghttp.h"
+#include <libgnomevfs/gnome-vfs.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
 #include <gdk/gdkx.h>
 
+#define STOCK_QUOTE(sq) ((StockQuote *)(sq))
 
 	GtkWidget *applet;
 	GtkWidget *label;
@@ -50,24 +51,23 @@
 	int location;
 	int MOVE;
 
-	char output[64];
+	typedef struct {
+		char *price;
+		char *change;
+		int color;
+	} StockQuote;
+	GArray *quotes;
 
-	/**
-	 * FOR COLOR
-	 * LEN and the length of output, and colorNum must match
-	 */
-	const int LEN = 20;
-	char outputArray[20][64];
-	char changeArray[20][64];
-	int colorArray[20];
-
-	const int RED = 1;
-	const int GREEN = 2;
+	enum {
+		WHITE,
+		RED,
+		GREEN
+	};
 
 	static const int max_rgb_str_len = 7;
 	static const int max_rgb_str_size = 8;
 
-	int setCounter, getCounter, setColorCounter,getColorCounter;
+	int setCounter;
 
 	GdkGC *gc;
 	GdkColor gdkUcolor,gdkDcolor;
@@ -81,7 +81,8 @@
 	/*  properties vars */
  	 
 	GtkWidget *tik_syms_entry;
-	gchar tik_syms[256];
+
+	GtkWidget *fontDialog = NULL;
 
 	GtkWidget * pb;
 	char buttons[16]="blank";
@@ -118,9 +119,6 @@
 	void removeSpace(char *buffer); 
 	int configured(void);
 	void timeout_cb( GtkWidget *widget, GtkWidget *spin );
-	static int http_get_to_file(gchar *a_host, gint a_port, 
-				gchar *a_resource, FILE *a_file);
-	int http_got(void);
 	void properties_save(char *path) ;
 	static void destroy_applet(GtkWidget *widget, gpointer data) ;
 	char *getSymsFromClist(GtkWidget *clist) ;
@@ -133,7 +131,6 @@
 	static void updateOutput(void) ;
 	static void reSetOutputArray(void) ;
 	static void setOutputArray(char *param1) ;
-	static void setColorArray(int theColor) ;
 	void setup_colors(void);
 	int create_gc(void) ;
 	void ucolor_set_cb(GnomeColorPicker *cp) ;
@@ -143,7 +140,7 @@
 
 	/* For fonts */
 	GdkFont * my_font;
-	gchar * new_font=NULL;
+	gchar * new_font = NULL;
 	GdkFont * extra_font;
 	GdkFont * small_font;
 	static gint symbolfont = 1;
@@ -185,30 +182,75 @@
 			g_error("Could not load fonts!");
 
 		if ( !extra_font || (strcmp(props.arrows,"noArrows")) == 0 ) {
-			gdk_font_unref(extra_font);
+			
+			if (extra_font)
+				gdk_font_unref(extra_font);
 			extra_font = gdk_font_load("fixed");
 			symbolfont = 0;
 		}
 		else {
-			gdk_font_unref(extra_font);
+			if (extra_font)
+				gdk_font_unref(extra_font);
 			extra_font = gdk_font_load ("-*-symbol-medium-r-normal-*-*-140-*-*-p-*-adobe-fontspecific");
 			symbolfont = 1;
 
 		}
 		if (!small_font) {
-			gdk_font_unref(small_font);
+			if (small_font)
+				gdk_font_unref(small_font);
 			small_font = gdk_font_load("fixed");
 		}
 	}
 
-	/*-----------------------------------------------------------------*/
-	static void updateOutput(void) {
-		if ( http_got() == -1 || !(configured()) ) {  
+/*-----------------------------------------------------------------*/
+static void xfer_callback (GnomeVFSAsyncHandle *handle, GnomeVFSXferProgressInfo *info, gpointer data)
+{
+	if (info->phase == GNOME_VFS_XFER_PHASE_COMPLETED) {
+		if (!configured()) {
 			reSetOutputArray();
 			fprintf(stderr, "No data!\n");
 			setOutputArray(_("No data available or properties not set"));
 		}
 	}
+}
+
+/*-----------------------------------------------------------------*/
+static void updateOutput(void)
+{
+	GList *sources, *dests;
+	GnomeVFSURI *source_uri, *dest_uri;
+	char *source_text_uri;
+	GnomeVFSAsyncHandle *vfshandle;
+
+	source_text_uri = g_strconcat("http://finance.yahoo.com/q?s=",
+				      props.tik_syms,
+				      "&d=v2",
+				      NULL);
+	source_uri = gnome_vfs_uri_new(source_text_uri);
+	sources = g_list_append(NULL, source_uri);
+	g_free(source_text_uri);
+
+	dest_uri = gnome_vfs_uri_new(configFileName);
+	dests = g_list_append(NULL, dest_uri);
+
+	if (GNOME_VFS_OK !=
+	    gnome_vfs_async_xfer(&vfshandle, sources, dests,
+				 GNOME_VFS_XFER_DEFAULT,
+				 GNOME_VFS_XFER_ERROR_MODE_ABORT,
+				 GNOME_VFS_XFER_OVERWRITE_MODE_REPLACE,
+				 (GnomeVFSAsyncXferProgressCallback) xfer_callback,
+				 NULL, NULL, NULL)) {
+		GnomeVFSXferProgressInfo info;
+		info.phase = GNOME_VFS_XFER_PHASE_COMPLETED;
+		xfer_callback(NULL, &info, NULL);
+	}	
+
+	g_list_free(sources);
+	g_list_free(dests);
+
+	gnome_vfs_uri_unref(source_uri);
+	gnome_vfs_uri_unref(dest_uri);
+}
 
 
 
@@ -222,8 +264,9 @@
 		   props.tik_syms = gnome_config_get_string("gtik/tik_syms");
 		
 
-		timeout = gnome_config_get_int("gtik/timeout") > 0 ? gnome_config_get_int ("gtik/timeout") : props.timeout;
-	
+		if (gnome_config_get_int("gtik/timeout") > 0)
+			props.timeout = gnome_config_get_int("gtik/timeout");
+		timeout = props.timeout;
 
 		if ( gnome_config_get_string ("gtik/output") != NULL ) 
 			props.output = gnome_config_get_string("gtik/output");
@@ -340,7 +383,7 @@
 		int  linenum=0;
 		int AllOneLine=0;
 		int flag=0;
-		char *section = NULL; /* FIXME: correct? this gets rid of warning */
+		char *section = NULL;
 		char *ptr;
 
 		if (strlen(line) > 64) AllOneLine=1;
@@ -397,17 +440,6 @@
 					strcpy(change,extractText(line));
 				else
 					strcpy(change,extractText(section));
-
-				if (strstr(change,"-")) {
-					setColorArray(RED);
-				}
-				else if (strstr(change,"+")) {
-					setColorArray(GREEN);
-				}
-				else {
-					setColorArray(0);
-				}
-				
 			}
 			else if (linenum == 4) {
 				if (!AllOneLine) 
@@ -444,7 +476,7 @@
 				fgets(buffer, sizeof(buffer)-1, CONFIG);
 
 				if (strstr(buffer,
-				    "<td nowrap align=left><a href=\"/q\?s=")) {
+				    "<td nowrap align=left><font face=arial size=-1><a href=\"/q\?s=")) {
 
 				      setOutputArray(parseQuote(CONFIG,buffer));
 				      retVar = 1;
@@ -464,83 +496,6 @@
 	}
 
 
-	/*-----------------------------------------------------------------*/
-	/* Shamelessly stolen from the Slashapp applet
-	 */
-	static int http_get_to_file(gchar *a_host, gint a_port, 
-				    gchar *a_resource, FILE *a_file) {
-		int length = -1;
-		ghttp_request *request = NULL;
-		gchar s_port[8];
-		gchar *uri = NULL;
-		gchar *body;
-		gchar *proxy = g_getenv("http_proxy");
-
-		g_snprintf(s_port, sizeof(s_port), "%d", a_port);
-		uri = g_strconcat("http://", a_host, ":", s_port, 
-						a_resource, NULL);
-
-#ifdef DEBUG
-		fprintf(stderr,"Asking for %s\n", uri);
-#endif
-
-		request = ghttp_request_new();
-		if (!request)
-			goto ec;
-		if (proxy && (ghttp_set_proxy(request,proxy) != 0))
-			goto ec;
-
-		if (ghttp_set_uri(request, uri) != 0)
-			goto ec;
-		ghttp_set_header(request, http_hdr_Connection, "close");
-		if (ghttp_prepare(request) != 0)
-			goto ec;
-		if (ghttp_process(request) != ghttp_done)
-			goto ec;
-		length = ghttp_get_body_len(request);
-		body = ghttp_get_body(request);
-		if (body != NULL)
-			fwrite(body, length, 1, a_file);
-
-		ec:
-			if (request)
-				ghttp_request_destroy(request);
-			if (uri)
-				
-				g_free(uri);
-		return length;
-	}
-
-
-
-
-	/*-----------------------------------------------------------------*/
-	int http_got() {
-
-		int retVar;
-		FILE *local_file;
-
-		char *tmpBuff;
-
-		tmpBuff = g_strconcat ("/q?s=", props.tik_syms, "&d=v2", NULL);
-
-		retVar = 0;
-
-		local_file = fopen(configFileName, "w");
-		retVar = http_get_to_file("finance.yahoo.com", 80, 
-						tmpBuff, local_file);
-
-		fclose(local_file);
-
-		g_free (tmpBuff);
-
-		return retVar;
-	}
-
-
-
-
-
 	/*-----------------------------------------------------------------*/
 	static gint expose_event (GtkWidget *widget,GdkEventExpose *event) {
 
@@ -603,14 +558,14 @@
 
 		for(i=0;i<setCounter;i++) {
 			totalLen += (gdk_string_width(my_font,
-					outputArray[i]) + 10);
+					STOCK_QUOTE(quotes->data)[i].price) + 10);
 			if (!strcmp(props.output,"default")) {
-				if (*(changeArray[i])) {
+				if (*(STOCK_QUOTE(quotes->data)[i].change)) {
 					totalLen += (gdk_text_width(extra_font,
-							changeArray[i],1) + 10);
+								    STOCK_QUOTE(quotes->data)[i].change,1) + 10);
 				}
 				totalLen += (gdk_string_width(small_font,
-						&changeArray[i][1]) +10 );
+						&STOCK_QUOTE(quotes->data)[i].change[1]) +10 );
 			}
 		}
 
@@ -646,35 +601,35 @@
 		for (i=0;i<setCounter;i++) {
 
 			/* COLOR */
-			if (colorArray[i] == GREEN) {
+			if (STOCK_QUOTE(quotes->data)[i].color == GREEN) {
 				gdk_gc_set_foreground( gc, &gdkUcolor );
 			}
-			else if (colorArray[i] == RED) {
+			else if (STOCK_QUOTE(quotes->data)->color == RED) {
 				gdk_gc_set_foreground( gc, &gdkDcolor );
 			}
 			else {
 				gdk_gc_copy( gc, drawing_area->style->white_gc );
 			}
 
-			tmpSym = outputArray[i];
+			tmpSym = STOCK_QUOTE(quotes->data)[i].price;
 			gdk_draw_string (pixmap,my_font,
 			gc,
-			location + totalLoc ,14,outputArray[i]);
+			location + totalLoc ,14,STOCK_QUOTE(quotes->data)[i].price);
 			totalLoc += (gdk_string_width(my_font,tmpSym) + 10); 
 
 
 			if (!strcmp(props.output,"default")) {
-				tmpSym = changeArray[i];
-				if (*(changeArray[i])) {
+				tmpSym = STOCK_QUOTE(quotes->data)[i].change;
+				if (*(STOCK_QUOTE(quotes->data)[i].change)) {
 					gdk_draw_text (pixmap,extra_font,
 					     gc, location + totalLoc,
-					     14,changeArray[i],1);
+					     14,STOCK_QUOTE(quotes->data)[i].change,1);
 					totalLoc += (gdk_text_width(extra_font,
-							changeArray[i],1) + 5);
+							STOCK_QUOTE(quotes->data)[i].change,1) + 5);
 				}
 				gdk_draw_string (pixmap,small_font,
 				     gc, location + totalLoc,
-				     14, &changeArray[i][1]);
+				     14, &STOCK_QUOTE(quotes->data)[i].change[1]);
 				totalLoc += (gdk_string_width(small_font,
 						tmpSym) + 10); 
 			}
@@ -698,7 +653,8 @@
 		static const gchar *authors[] = {
 			"Jayson Lorenzen <jayson_lorenzen@yahoo.com>",
 			"Jim Garrison <garrison@users.sourceforge.net>",
-			"Rached Blili <striker@dread.net>"
+			"Rached Blili <striker@dread.net>",
+			NULL
 		};
 
 		about = gnome_about_new (_("The GNOME Stock Ticker"), VERSION,
@@ -844,6 +800,13 @@
 		if (strcmp(poutput,"blank") !=0) {
 			props.output = g_strdup(poutput);
 		}
+
+
+		if (updateTimeID > 0)
+			gtk_timeout_remove(updateTimeID);
+		updateTimeID = gtk_timeout_add(props.timeout * 60000,
+				   (gpointer)updateOutput,"NULL");
+
 		properties_save(APPLET_WIDGET(applet)->privcfgpath);
 		properties_set(TRUE);
 	}
@@ -860,24 +823,23 @@
 
 	/*-----------------------------------------------------------------*/
         static gint font_selector( GtkWidget *widget, void *data ) {
-                GtkWidget *tmpWidget;
-                GtkFontSelectionDialog *fontDialog;
-
-                tmpWidget = gtk_font_selection_dialog_new("Font Selector");
-                fontDialog = GTK_FONT_SELECTION_DIALOG(tmpWidget);
-		gtk_signal_connect (GTK_OBJECT (fontDialog ->ok_button),
-					"clicked",
-					GTK_SIGNAL_FUNC(OkClicked),fontDialog);
-                gtk_signal_connect_object(GTK_OBJECT(fontDialog->cancel_button),
-					"clicked",
-                                        GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                                        GTK_OBJECT(fontDialog));
-                gtk_signal_connect (GTK_OBJECT (fontDialog), "destroy",
-                                        GTK_SIGNAL_FUNC(QuitFontDialog),
-					fontDialog);
-
-                gtk_widget_show(tmpWidget);
-
+		if (!fontDialog) {
+			fontDialog = gtk_font_selection_dialog_new("Font Selector");
+			gtk_signal_connect (GTK_OBJECT (GTK_FONT_SELECTION_DIALOG(fontDialog)->ok_button),
+					    "clicked",
+					    GTK_SIGNAL_FUNC(OkClicked),fontDialog);
+            		gtk_signal_connect_object(GTK_OBJECT(GTK_FONT_SELECTION_DIALOG(fontDialog)->cancel_button),
+						  "clicked",
+						  GTK_SIGNAL_FUNC(gtk_widget_destroy),
+						  GTK_OBJECT(fontDialog));
+            		gtk_signal_connect (GTK_OBJECT (fontDialog), "destroy",
+					    GTK_SIGNAL_FUNC(QuitFontDialog),
+					    fontDialog);
+			
+            		gtk_widget_show(fontDialog);
+		} else
+			gdk_window_raise(fontDialog->window);
+			
 		return FALSE;
 	}
 
@@ -912,6 +874,7 @@
 
         /*-----------------------------------------------------------------*/
         gint QuitFontDialog( GtkWidget *widget, void *data ) {
+		fontDialog = NULL;
 		return FALSE;
         }
 
@@ -968,8 +931,6 @@
 		if ((temp=strtok(syms,"+")))
 			symbol[0] = g_strdup(temp);
 
-		g_free(syms);
-		
 		while (symbol[0]) {
 			gtk_clist_append(GTK_CLIST(clist),symbol);
 			if ((temp=strtok(NULL,"+")))
@@ -982,7 +943,6 @@
 			symbol[0] = g_strdup(temp);
 			gtk_clist_append(GTK_CLIST(clist),symbol);
 		}
-		g_free(temp);		
 	}
 
 	/*-----------------------------------------------------------------*/
@@ -1019,23 +979,12 @@
 	}
 
 	static void remFromClist(GtkWidget *widget, gpointer data) {
-		GList *selection, *seliter;
-		gint row;
-		GtkWidget *clist;
+		GtkCList *clist;
 
-		clist = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(widget),
-							"clist"));
-		selection = g_list_copy (GTK_CLIST(clist)->selection);
-
-		seliter = selection;
-
-		while(seliter != NULL) {
-			row=GPOINTER_TO_INT(seliter->data);
-			seliter = seliter->next;
-			gtk_clist_remove(GTK_CLIST(clist),row);
-		}
-
-		g_list_free (selection);
+		clist = GTK_CLIST(gtk_object_get_data(GTK_OBJECT(widget),
+						      "clist"));
+		while (GPOINTER_TO_INT(clist->selection))
+			gtk_clist_remove(GTK_CLIST(clist), GPOINTER_TO_INT(clist->selection->data));
 	}
 
 	static GtkWidget *symbolManager() { 
@@ -1176,8 +1125,7 @@
 
 		tik_syms_entry = gtk_entry_new_with_max_length(60);
 
-		/* tik_syms var is here if you want a default value */
-		gtk_entry_set_text(GTK_ENTRY(tik_syms_entry), props.tik_syms ? props.tik_syms : tik_syms);
+		gtk_entry_set_text(GTK_ENTRY(tik_syms_entry), props.tik_syms ? props.tik_syms : "");
 		gtk_signal_connect_object(GTK_OBJECT(tik_syms_entry), "changed",GTK_SIGNAL_FUNC(changed_cb),GTK_OBJECT(pb));
 
 		check = gtk_check_button_new_with_label(_("Display only symbols and price"));
@@ -1318,18 +1266,6 @@
 
 
 
-	/*-----------------------------------------------------------------*/
-	/* Do the first update in an idle since we block (UGLY UGLY UGLY)
-	 * and thus this is at least marginally better behaviour since the
-	 * user will at least see something on the screen */
-	static gboolean
-	first_update (gpointer data)
-	{
-		updateOutput ();
-		return FALSE;
-	}
-
-
 	/*-----------------------------------------------------------------*/
 	int main(int argc, char **argv) {
 		GtkWidget * vbox;
@@ -1346,6 +1282,7 @@
 		call gnome_init */
 		applet_widget_init(GTIK_APPLET_NAME, VERSION, argc, argv,
 				    NULL, 0, NULL);
+		gnome_vfs_init();
 
 		/* create a new applet_widget */
 		applet = applet_widget_new(GTIK_APPLET_NAME);
@@ -1354,7 +1291,7 @@
 		if (!applet)
 			g_error("Can't create applet!\n");
 
-	
+		quotes = g_array_new(FALSE, FALSE, sizeof(StockQuote));	
 
 		vbox = gtk_hbox_new (FALSE,0);
 		leftButton = gtk_button_new_with_label("<<");
@@ -1451,7 +1388,7 @@
 			gtk_widget_show(rightButton);
 		}
 
-		g_idle_add (first_update, NULL);
+		updateOutput();
 
 		/* special corba main loop */
 		applet_widget_gtk_main ();
@@ -1478,25 +1415,15 @@
 /*HERE*/
 	/*-----------------------------------------------------------------*/
 	static void reSetOutputArray() {
-		int i;
-		
-		for (i=0;i<LEN;i++) {
-			/* CLEAR EACH SYMBOL'S SPACE */
-			memset(outputArray[i],0,sizeof(outputArray[i]));
-
-			/* CLEAR ASSOC COLOR ARRAY */
-			colorArray[i] = 0;
-
-			/* CLEAR ADDITIONAL INFO */
-			memset(changeArray[i],0,sizeof(changeArray[i]));
 
+		while (quotes->len) {
+			int i = quotes->len - 1;
+			g_free(STOCK_QUOTE(quotes->data)[i].price);
+			g_free(STOCK_QUOTE(quotes->data)[i].change);
+			g_array_remove_index(quotes, i);
 		}
 
 		setCounter = 0;
-		getCounter = 0;
-		setColorCounter = 0;
-		getColorCounter = 0;
-		
 	}	
 
 
@@ -1554,29 +1481,35 @@
 
 	/*-----------------------------------------------------------------*/
 	static void setOutputArray(char *param1) {
+		StockQuote quote;
 		char *price;
 		char *change;
 
 		price = splitPrice(param1);
 		change = splitChange(param1);
 
-		if (setCounter < LEN) {
-			
-			strcpy(outputArray[setCounter],price);
-			strcpy(changeArray[setCounter],change);
-		}
+		quote.price = g_strdup(price);
+		quote.change = g_strdup(change);
+
+		if (strstr(change,"-"))
+			quote.color = RED;
+		else if (strstr(change,"+"))
+			quote.color = GREEN;
+		else
+			quote.color = WHITE;
+
+#if 0
+		g_message("Param1: %s\nPrice: %s\nChange: %s\nColor: %d\n\n", param1, price, change, quote.color);
+#endif
+
+		g_array_append_val(quotes, quote);
+
 		setCounter++;
 	}
 
 
 
 	/*-----------------------------------------------------------------*/
-	static void setColorArray(int theColor) {
-		if (setColorCounter < LEN) {
-			colorArray[setColorCounter] = theColor;
-		}
-		setColorCounter++;
-	}
 
 	void setup_colors(void) {
 		GdkColormap *colormap;
